UNIT vmdomus0; {Jul 11 2002}

INTERFACE
USES Windows, Dialogs, SysUtils, vzlocom, varfun, vmria0, vmria3;

TYPE HKLDREC = RECORD h,k,l:INTEGER; d:REAL;
               END;
CONST i32 :LONGINT = 32768;
      tofcon=505.556{2.*252.778};
CONST inicomm:ARRAY[1..25] OF STRING[47] = (
 '...Beginning of the 3-dim.spectrum N-coordinate',
 '...End of the 3-dim.spectrum N-coordinate',
 '...Beginning of the 3-dim.spectrum X-coordinate',
 '...End of the 3-dim.spectrum X-coordinate',
 '...Beginning of the 3-dim.spectrum Y-coordinate',
 '...End of the 3-dim.spectrum Y-coordinate',
 '...Flight Path',
 '...DELAY',
 '...Channel Width (mks)',
 '...2*THeta0',
 '...DthX(m)',
 '...DthY(m)',
 '...L Constant (m)',
 '...Lambda Smoothing',
 '...SENSitivity',
 '...RESolution',
 '...WNmaX',
 '...WXmaX',
 '...WYmaX',
 '...WNmaN',
 '...WXmaN',
 '...WYmaN',
 '...HVMin',
 '...NX in D-scale',
 '...NY in D-scale'                        );
CONST nspi:LONGINT = 262144;
VAR
  ft:TEXTFILE;
  l:LONGINT;
  x,y,z,f:REAL;
  iw,ih,it,ix:LONGINT;        {iwidth, iheight etc. of Image2}
  fnhkli,fnhklo:STRING;       {File names for hkl in/out-put files}
TYPE TVDomus = CLASS
  PUBLIC { Public declarations }
  sfn,esfn:STRING;           {Spectrum and Effec.Spectrum file names}
  sp:ARRAY[1..1024,1..32,1..8] OF INTEGER;  {Spectrum}
  ssp:ARRAY[1..1024,1..32,1..8] OF REAL;    {Smoothed Spectrum}
  hsp:ARRAY[1..342,1..16,1..8] OF INTEGER;  {Auxi.Spectrum}
  esp:ARRAY[1..1024,1..32,1..8] OF INTEGER; {Effective Spectrum}
  csp:ARRAY[1..1024] OF REAL;      {XY-Convoluted spectrum}
  nsecar:ARRAY[1..1024] OF REAL;   {N-Section}
  snx:ARRAY[1..1024,1..32] OF INTEGER;{Y-Convoluted spectrum}
  sny:ARRAY[1..1024,1..8] OF INTEGER; {X-Convoluted spectrum}
  i1,i2,j1,j2,k1,k2:INTEGER;  {Bounds of a 3-dimensional spectrum}
  mN1,mN2,mx1,mx2,my1,my2:INTEGER; {Bounds of the reversed spectrum}
  spx,snor:REAL;              {Current and normal max in the spectrum}
  cspx,csnor:REAL;            {Current and normal max in XY-conv.sp.}
  cxnx,cynor:REAL;            {Current and normal max in Y-conv.sp.}
  cxny,cxnor:REAL;            {Current and normal max in X-conv.sp.}
  espx:REAL;                  {Max in the effictive spectrum}
  aspx:REAL;                  {Max in the additional spectrum}
  rl,sens,res:REAL;           {smoothing, sensitivity, resolution}
  wx,wy,wz,                   {peak widths}
  wxxl,wyyl,wzzl:REAL;        {peak width lower bounds}
  wxxu,wyyu,wzzu:REAL;        {peak width upper bounds}
  hvmin:REAL;                 {minimal hypervolume}
  npeak:INTEGER;              {number of found peaks}
  parr:ARRAY[1..7,1..512] OF REAL;{peak array: x,y,z,A,wx,wy,wz etc.}
  pari:ARRAY[1..7,1..512] OF REAL;{peak array in the reversed space}
  hkldar:ARRAY[1..1024] OF HKLDREC;
  Fp:REAL;                    {flight path}
  delay:REAL;                 {delay}
  chw:REAL;                   {channel width}
  Bra:REAL;                   {double Bragg angle 0}
  dthx:REAL;                  {delta thx}
  dthy:REAL;                  {delta thy}
  lm:REAL;                    {additional small flight path}
  sum:REAL;                   {sum of peak areas}

  noefs:BOOLEAN;              {is or not the effective spectrum here}
  noefsc:BOOLEAN;             {Corrected/not for the effective spectrum}
  norevers:BOOLEAN;           {Was the spectrum reversed?}
  noToF:BOOLEAN;              {is/not the TOF information}

  xgp,ygp:INTEGER;            {x,y of the N-distribution in 3-d.spectr}
  ddev:REAL;                  {admissible deviation for d}
  CONSTRUCTOR Create;
  PROCEDURE ComputeXYZ(ic:LONGINT; VAR i,j,k:INTEGER);
  PROCEDURE computeD(ch,x,y:REAL; VAR a,th,d:REAL);
  FUNCTION CompCurrMax(snor:REAL):REAL;
  PROCEDURE SetTheMaxima;
  PROCEDURE ConvoluteTheSpectrum(VAR csnor:REAL; m:INTEGER);
  PROCEDURE ReadTheSpectrum;
  PROCEDURE fismax3(m:INTEGER);
  PROCEDURE Convolute;
  PROCEDURE Read3Binary;
  PRIVATE { Private declarations }
  END;
IMPLEMENTATION
CONSTRUCTOR TVDomus.Create;
BEGIN i1:=1; i2:=1024; j1:=1; j2:=32; k1:=1; k2:=8;
mN1:=1; mN2:=1024; mx1:=1; mx2:=32; my1:=1; my2:=8;
xgp:=(j1+j2)DIV 2; ygp:=(k1+k2)DIV 2; sfn:=''; esfn:='';
rl:=0.5; sens:=1; res:=1;
wxxl:=7; wyyl:=4; wzzl:=1.5; wxxu:=10; wyyu:=5; wzzu:=3; hvmin:=1000;
Fp:=23.75; delay:=-30; chw:=64; Bra:=88;
dthx:=0.0075; dthy:=0.02; lm:=0.51;
trimm:=0.50; grst:=0;
noefs:=true; noefsc:=true; noToF:=true; norevers:=true;
ncl:=0;
butar:='1111111111'; lhelp:=0;
END;
PROCEDURE TVDomus.ComputeXYZ(ic:LONGINT; VAR i,j,k:INTEGER);
VAR il,ii:LONGINT;
BEGIN il:=((ic-1) DIV i32)+1; k:=il;
ii:=ic-1-(k-1)*i32; il:=(ii DIV 1024)+1; j:=il;
il:=ii-(j-1)*1024+1; i:=il;
END;
PROCEDURE TVDomus.computeD(ch,x,y:REAL; VAR a,th,d:REAL);
{***  Purpose: To compute dk ***  Version: Feb 16 1999 }
CONST jm:INTEGER = 32;
      km:INTEGER = 8;
VAR   pi2,x0,dx,dy,dxx,ang,ca,b,c,dl,rmax,xx,s,sinth:REAL;
      dthxr,dthyr,theta0:REAL;
BEGIN pi2:=pi*0.5; x0:=jm*0.5;
theta0:=Bra/radian; dthxr:=dthx/radian; dthyr:=dthy/radian;
dx:=(x-jm*0.5)*dthxr; dy:=(y-km*0.5)*dthyr; dxx:=x0*dthxr;
ang:=pi2-theta0; IF(theta0 > pi2)THEN ang:=theta0-pi2;
ca:=COS(ang); b:=dxx/ca; dl:=b*SIN(ang); rmax:=lm-dl;
IF(theta0 > pi2)THEN rmax:=lm+dl; dxx:=ABS(x)*dthxr;
{ Determine a,c }
c:=SQRT(SQR(dy)+SQR(dxx)+SQR(rmax)); a:=SQRT(SQR(dx)+SQR(dy)+SQR(lm));
xx:=(SQR(c)+SQR(b)-SQR(a))/(2*c); s:=c*SQRT(SQR(b)-SQR(xx))*0.5;
sinth:=2*s/(a*b); th:=ARCSIN(sinth);
IF(c > SQRT(SQR(a)+SQR(b)))THEN th:=pi-th;
th:=th*0.5; d:=(ch+delay)/(tofcon*(Fp+a)/chw*SIN(th));
END;
FUNCTION TVDomus.CompCurrMax(snor:REAL):REAL;
VAR spx:REAL;
BEGIN
CASE nscale OF
1: spx:=snor;
2: spx:=SQRT(ABS(snor));
3: spx:=LN(ABS(snor)+1);
4: spx:=snor*trimm;
ELSE spx:=1; END;
CompCurrMax:=spx;
END;
PROCEDURE TVDomus.SetTheMaxima;
BEGIN spx:=CompCurrMax(snor); cspx:=CompCurrMax(csnor);
cxnx:=CompCurrMax(cxnor); cxny:=CompCurrMax(cynor);
END;
PROCEDURE TVDomus.ConvoluteTheSpectrum(VAR csnor:REAL; m:INTEGER);
VAR i,j,k:INTEGER; l:LONGINT; t:REAL;
BEGIN csnor:=0; FOR i:=i1 TO i2 DO
 BEGIN CASE m OF
 0: BEGIN t:=0; FOR j:=j1 TO j2 DO FOR k:=k1 TO k2 DO t:=t+sp[i,j,k];
    csp[i]:=t; IF(t>csnor)THEN csnor:=t;
    END;
 1: FOR j:=j1 TO j2 DO
     BEGIN t:=0; FOR k:=k1 TO k2 DO t:=t+sp[i,j,k];
     snx[i][j]:=Round(t); IF(t>csnor)THEN csnor:=t;
     END;
 2: FOR k:=k1 TO k2 DO
     BEGIN t:=0; FOR j:=j1 TO j2 DO t:=t+sp[i,j,k];
     sny[i][k]:=Round(t); IF(t>csnor)THEN csnor:=t;
     END;
       ELSE END;
 END;
END;
PROCEDURE TVDomus.ReadTheSpectrum;
BEGIN WITH Form2 DO
 BEGIN OpenDialog1.Title:='   Select the file';
 OpenDialog1.FileName:=sfn;
 IF(OpenDialog1.Execute)THEN sfn:=OpenDialog1.FileName ELSE Exit;
 IF(NOT FileExists(sfn))THEN ShowMessage('No file '+sfn)
                        ELSE
  BEGIN Read3Binary; IF(spx=0)THEN
   BEGIN ShowMessage('++++ Error, the spectrum is zero-valued'); Exit;
   END                        ELSE WITH Canvas DO
    BEGIN ShowMessage('The spectrum '+sfn+' read in');
    Convolute; SetTheMaxima;
    npeak:=0; noefsc:=true; norevers:=true; nosavedD:=false;
    END;
  END;
 END;
END;
PROCEDURE TVDomus.fismax3(m:INTEGER);
{***  Purpose: To find the maximum of the spectrum Version: Jan 6 2000}
VAR i,j,k:INTEGER; sn,r:REAL;
BEGIN sn:=-1E10;
FOR k:=k1 TO k2 DO FOR j:=j1 TO j2 DO FOR i:=i1 TO i2 DO
 BEGIN CASE m OF
 0: r:=sp[i,j,k];
 1: r:=esp[i,j,k];
 2: r:=ssp[i,j,k];
 ELSE END;
 IF(r > sn)THEN sn:=r;
 END;
CASE m OF
0: snor:=sn;
1: espx:=sn;
2: aspx:=sn;
ELSE END;
spx:=CompCurrMax(snor);
END;
PROCEDURE TVDomus.Convolute;
BEGIN ConvoluteTheSpectrum(csnor,0); ConvoluteTheSpectrum(cynor,1);
ConvoluteTheSpectrum(cxnor,2);
END;
PROCEDURE TVDomus.Read3Binary;
{Purpose: To read in binary 3-dim files, Dec 3 1999}
VAR  res,i,j,k,ir,ii:INTEGER; sum,r:REAL; fb:FILE OF INTEGER;
     kk,il:LONGINT;
BEGIN AssignFile(fb,sfn); Reset(fb);
FOR ir:=1 TO 1024 DO
 BEGIN FOR j:=1 TO 256 DO buf2[j]:=0; BlockRead(fb,buf2,128,res);
 kk:=(ir-1)*256; FOR ii:=1 TO 256 DO
  BEGIN il:=kk+ii; ComputeXYZ(il,i,j,k); sp[i,j,k]:=buf2[ii];
  END;
 END;
CloseFile(fb);
fismax3(0);
END;
{----------}
BEGIN
END.
