UNIT vmriarit;
INTERFACE
USES Windows,  Messages, Classes, Forms,    StdCtrls, ExtCtrls, Grids,
     Graphics, comctrls, Dialogs, SysUtils, Controls, vzlocom,  varfun,
     vmria0,   vmria2,   vgfit,   vmriahkl, vspectr,  vpmodel,  vpeaksr,
     vstdcons, vmria3,   vmria00;

PROCEDURE armrit(VAR yy,w,v:REAL; jp,typ:INTEGER);
PROCEDURE ComputeQs(i0,iph,iref:INTEGER; VAR q,qold:REAL);
PROCEDURE coextc(pj0,theta:REAL; iref,iph:INTEGER; VAR Yp,dFdrho:REAL);
PROCEDURE ProcessBackground(ibk:INTEGER; VAR backv:REAL; d,xm:REAL);
PROCEDURE FkDerivatives(iph,iref,i0:INTEGER; akcm,Fkc,Fks,pt,Fkm:REAL);
PROCEDURE regmria(iit:INTEGER);
PROCEDURE MakeFitResLB(VAR i:INTEGER);
PROCEDURE GetSingleFk(j,iph:INTEGER; VAR rc,rs:REAL);
PROCEDURE arpeak(VAR yy,w,v:REAL; jp,typ:INTEGER);
PROCEDURE MakeBMGraph(amx0,amn0:REAL; indi:STRING);
PROCEDURE UpdateOldCell(iph:INTEGER);
PROCEDURE ConExAr(VAR yy,ww,v:REAL; jp,typ:INTEGER);


IMPLEMENTATION
PROCEDURE armrit(VAR yy,w,v:REAL; jp,typ:INTEGER);
{Purpose: arithmetic block for diffraction spectrum analysis
 -----------------------------------------
 y(x) - observed neutron spectrum in interval [x1,x2]:
                      nph
 y(x) := exp(-ac*x) * SUM c_i Ph_i (x)  +  back(x)   ,   where
                      i=1
 x := channel in [x1,x2], Ph - phase counts, back - background,
 ac := correcting parameter.
           nhkl                  x-x_k
   Ph(x) = SUM a_k mod(F_k)^2 m(-------) ,  nhkl - n of hkls,
           k=1                  W0+W1*x
           nat
   F_k  =  SUM o_j b_j exp(6.28i(k,x_j)-(B_j*k,k) ),      k = (hkl),
           j=1
 nat - n of atoms, o_j = occupation, bj = scattering amplitude
 Bj = thermal tensor (B11,B12,B13,B22,B23,B33)
 in isotropic case (B,0,0,0,0,0) B=b11*qold*.25
 a_k = multiplicities, corrected for Lorentz factor: dk^4 and extinc.ext
 x_k = const*L*sin(theta+theta0) * dk - x0;   const = 2*252.778/cw,
 cw    = channel width (in mksec),
 L     = time of flight base(m)
 theta = Bragg angle(deg)
 dk = 1/SQRT(A*k,k),  (A*k,k) = (a11h+a12k+a13l)h+(a22k+a23l)k+a33ll
 Cell parameters(a11,a22,a33,a23,a13,a12) are =
                    a11=a*^2  a22=b*^2  a33=c*^2
    a12=2a*b*cos_gamma*  a13=2a*c*cos_beta*  a23=2b*c*cos_alpha*
 (a*,b*,c*,alpha*,beta*,gamma*- reciprocal unit cell parameters)
                           x-xb0
 back(x) := P3(Z)^2 * m0 (---------)  +  b0
                          Wb0+Wb1*x

 P3(Z) := (b1 + b2*Z^2 ) ,  Z = (2*x-x1-x2)/(x2-x1)
 parameters are located: b0,b1,b2,xb0,Wb0,Wb1

 or back(x) is a sum of orthogonal polynomials of degree < 6.
 -------------------------
 parameters here :

 common:   L,teta0,x0,b0,b1,b2,xb0,Wb0,Wb1,ac               10
 for each phase: c,a11,a22,a33,a12,a13,a23,W0,W1,ext        10
 for each atom:  o,x,y,z.B11,B12,B13,B22,B23,B33;           10
 for each phase: if magnetic, phia phib phic                 3
 for each phase: if anisotropic,  ani_x,ani_y,any_z          3
 [for each addi-spectrum: L,teta0,x0,b0,b1,b2,b3,b4,b5,ac   10
  for each phase: c,W0,W1]                                   3

 Location: 10 common + nph*(10 + nat*10) + nph*3 (mg) + nph*3 (an) +
          [(10+3*nph)*(ns-1)]
 }

{***  Version: Fortran: July 13 1994  Oct 21 1997,
               delphi:  Sep 28 2000   Oct 10 2003

{ arithmetic block for Powder Match and Fourier synthesis

         -ac*x     nph
 y(x) = e      *   SUM c Ph (x)  +  back(x)   ,   where
                   i=1  i  i

 x = x-channel, PH - phase, back - background,
 ac = correcting parameter for effective spectrum.

  Version: Aug 2 1995
  form-factor
 for texture coefficients: 1+t*k/(k+eps), k:=hkl
 (for 1 spectrum only:  wcw2  txh   txk   txl                  4
  }

VAR   i,ip,iph,iref,nref,natom,j0,ii,ksp,iswtch:INTEGER;
      npa2,iw,icp,iwd,i0,im2:INTEGER;
      mn,ht,kt,lt,r,cpw,thc,theta0,d,xcor,sumph,Lpath,D0,acor,sinT:REAL;
      d3,d5,C252,cPathS,fcor,q,q12,w0,w1,w2,cph,phsum:REAL;
      fq2,Fk2,Fc,Fs,Fm,Dkk,akcm,Yp,dFdrho:REAL;
      ptot,backv,x1,x2,cnst,z,ff,fz,fzz:REAL;
      am2,th2,wcc,wc,wlat,zn,xm2,xm,xb:REAL;
      facL,ania,anic,qanis:REAL;
      ok,tofa:BOOLEAN;
      awa,aw2:ARRAY[1..3] OF REAL; aww:ARRAY[1..6] OF REAL;
LABEL L2;
BEGIN ksp:=numsp; ip:=jp+spec[ksp].a11-1; d:=ip;
IF(typ=4)THEN
 BEGIN ksp:=Round(sarg); d:=xarg;
 END;
ip:=Trunc(d);

{R-factors=0} IF((j=1)AND(typ=3))THEN
 BEGIN frp[ksp]:=0; frb[ksp]:=0; frw[ksp]:=0;
 fre[ksp]:=0; fie[ksp]:=0; fi2[ksp]:=0;
 END;

yy:=spec[ksp].y[ip]; w:=spec[ksp].e[ip]; IF(w<>0)THEN w:=1/(w*w);
FOR i:=1 TO gfit.npar DO gfit.par[i].drv:=0; tofa:=spec[ksp].tofang=0;
x1:=spec[ksp].a11; x2:=spec[ksp].a22; xm:=(2*d-x1-x2)/(x2-x1);
xcor:=(d-x1+1)/(x2-x1+1); xcor:=EXP(xcor); xm2:=xm; xb:=xm;
IF(keyWdt=1)THEN xm2:=(xm2+1)*(xm2+1)/4;
sumph:=0; j0:=0; WITH gfit DO
 BEGIN Lpath:=par[j0+1].p; theta0:=spec[ksp].theta+par[j0+2].p;{Th+th0}
 D0:=par[j0+3].p; acor:=par[j0+10].p;
 END;
fcor:=powerRR(xcor,acor); sinT:=SIN((theta0)/radian); npa2:=npani-9;
WITH spec[ksp] DO
 CASE tofang OF
 0: BEGIN C252:=tofcon/chw; cPathS:=C252*Lpath*sinT;
    END;
 1: BEGIN cPathS:={lambda}Lpath; thc:=x[ip]/2/radian;
    xm:=COS(thc); IF(xm>0)THEN xm:=SIN(thc)/xm;
    theta0:=SIN(thc)*SIN(2*thc); xm2:=xm;
    END;
 ELSE END;
im2:=GetNAddiPar(2);{for m2-mode}
am2:=0; IF(use_m2)THEN WITH gfit DO am2:=par[im2+1].p+par[im2+2].p*xm;
            {BACKGROUND AND ITS DERIVATIVES}
ProcessBackground(j0,backv,d,xb);

                    {PHASE  LOOP}
FOR iph:=1 TO nph DO
 BEGIN nref:=lat[iph].nhkl; natom:=lat[iph].nat;
 iswtch:=iph; IF(phassp=1)THEN iswtch:=ksp;
 {i0 - ith phase, kspth spec. icp,iwd - phase+width location}
 i0:=Geti0(iph); icp:=i0; iwd:=icp+8;
 iw:=GetNAddiPar(1)+(iph-1)*npani;{for aniso width}
 WITH gfit DO
  BEGIN cph:=par[icp+1].p*fcor; w0:=par[iwd].p;
  CASE tofa OF
  true: BEGIN w1:=par[iwd+1].p; w2:=0; xm2:=0;
        END;
  ELSE w1:=par[iwd+1].p; w2:=par[iwd+2].p;
      END;
  END;
                     {REFLECTION loop}
 phsum:=0; FOR iref:=1 TO nref DO
  BEGIN ComputeQs(i0,iph,iref,q,q_old); q12:=SQRT(ABS(q));
  IF(q12<0.0000001)THEN q12:=0.0000001;
  q_old:=AMAX1(q_old,0.0000001);
  IF((jp<=gfit.cb.skip)AND(mriamode=0))THEN
   WITH hkla[iref,iph] DO{get struc.factors, if 1st point}
    BEGIN StructFactor(1,iph,h,k,l,mul,i0,Fc,Fs,Fm);
    Fkc:=Fc; Fks:=Fs; Fkm:=Fm;
    END;
  {Correct for the texture if needed}
  WITH hkla[iref,iph] DO {form the Lorentz factor}
   BEGIN Fc:=Fkc; Fs:=Fks; Fm:=Fkm; ptot:=SQRT(ABS(txtc));
   IF(tofa)THEN facL:=1/(SQR(q_old))
           ELSE
            BEGIN facL:=theta0; IF(facL>0.000001)THEN facL:=1/theta0;
            END;
   facL:=ABS(facL)*ptot; mn:=mul;
   END;
                  {MAKE WIDTH}
  d5:=SQR(q_old)*SQRT(q_old); d3:=q_old*SQRT(q_old);
  CASE keyWdt OF
  0,1: zn:=AMAX1(w0+w1*xm+w2*SQR(xm2),0.001);
  2: zn:=SQRT(AMAX1(w0+w1/q_old,0.001));
  ELSE END;

  ania:=1; anic:=0; qanis:=0;
  IF(anis_mod)THEN WITH gfit DO WITH hkla[iref,iph] DO{anisotr-mode}
   BEGIN r:=lat[iph].hklx4*3;
   FOR i:=1 TO 3 DO awa[i]:=0;
   FOR i:=1 TO 6 DO aww[i]:=0; FOR i:=1 TO 3 DO aw2[i]:=0;
   WITH lat[iph] DO FOR i:=1 TO nut DO
    BEGIN ht:=ut[1,i]*h+ut[4,i]*k+ut[7,i]*l;
    kt:=ut[2,i]*h+ut[5,i]*k+ut[8,i]*l;lt:=ut[3,i]*h+ut[6,i]*k+ut[9,i]*l;
    aww[1]:=aww[1]+SQR(ht*ht)/r; aww[2]:=aww[2]+SQR(kt*kt)/r;
    aww[3]:=aww[3]+SQR(lt*lt)/r; aww[4]:=aww[4]+SQR(ht*kt)/r;
    aww[5]:=aww[5]+SQR(ht*lt)/r; aww[6]:=aww[6]+SQR(kt*lt)/r;
    aw2[1]:=aw2[1]+SQR(ht)*ht*kt/r; aw2[2]:=aw2[2]+SQR(kt)*kt*ht/r;
    aw2[3]:=aw2[3]+SQR(lt)*ht*kt/r;
    END;
   {IF(spec[numsp].tofang=0)THEN}
   FOR i:=1 TO 3{6} DO aww[i]:=aww[i]/d5;
   FOR i:=1 TO 3 DO
    BEGIN awa[i]:=aww[i]; IF(mriamode=2)THEN awa[i]:=0;
    aw2[i]:=aw2[i]/d3;
    END;
   FOR i:=1 TO 3 DO ania:=ania+gfit.par[iw+i].p*awa[i];
   {IF(mriamode=0)THEN ania:=ania/zn;}
   {DerivChan_d(q12,fq2,cPathS,ksp,dr); GetAnisoDers(iref,iph,sda);}
   FOR i:=1 TO 6 DO qanis:=qanis+par[iw+3+i].p*aww[i];
   FOR i:=1 TO npa2 DO qanis:=qanis+par[iw+9+i].p*aw2[i];
   zn:=AMAX1(zn+qanis,0.001);
   END;
                  {PEAK CENTER}
  WITH spec[ksp] DO
   IF(tofa)THEN Dkk:=cPathS/(q12)-D0 {Dk}
           ELSE                      {angles}
    BEGIN th2:=2*radian*arcsin(AMIN1(lambda*q12*0.5,1))-D0; {2theta}
    Dkk:=celx1*th2+celx0;  {channel}
    fq2:=AMIN1(cPathS*q12*0.5,0.9999); fq2:=SQRT(1-SQR(fq2));
    END;
                  {PEAK MODEL VALUE}

  z:=(d-Dkk)/zn; ff:=0; fz:=0; fzz:=0;
  WITH pmod[iph] DO
   BEGIN IF(ABS(z)>=rl)THEN GOTO L2; Getffzfzz(z,ff,fz,fzz);
   END;
  IF(use_m2)THEN anic:=am2*fzz; akcm:=cph*mn*facL*ania*(ff+anic);
          {DERIVATIVES WITH RESPECT TO FK-PARAMETERS}
              {ALSO WITH RESPECT TO MAGNETICS}
  CASE mriamode OF
  0: BEGIN FkDerivatives(iph,iref,i0,akcm,Fc,Fs,ptot,Fm);
     Fk2:=Fc*Fc+Fs*Fs; Fk2:=Fk2*SQR(Fm);
      {Extinction correction}
     WITH gfit DO WITH par[i0+10] DO IF(pl<>pu)THEN
      BEGIN coextc(par[i0+10].p,theta0,iref,iph,Yp,dFdrho);
      Fk2:=Yp*Fk2; par[i0+10].drv:=par[i0+10].drv+dFdrho*cph*2*ff*Fk2;
      FOR i:=i0+11 TO i0+10+natom*10 DO par[i].drv:=par[i].drv*Yp;
      END;
     END;
  2: Fk2:=gfit.par[i0+10+iref].p;
  ELSE END;
          {CONSTANTS wcc and wlat}
  WITH spec[ksp] DO
   BEGIN wcc:=cph*mn*facL*Fk2*ania*fz/zn;
   CASE tofa OF
   true:  wlat:=wcc*cPathS*0.5/(q12*q12*q12);       {ToF}
   false: wlat:=-wcc*0.5*radian*celx1*cPathS/(q12*fq2);{Ang}
   ELSE END;
   END;
          {DERIVATIVES WITH RESPECT TO CELL PARAMETERS}
  WITH gfit DO WITH hkla[iref,iph] DO
   BEGIN par[i0+2].drv:=par[i0+2].drv+wlat*(h*h);
   par[i0+3].drv:=par[i0+3].drv+wlat*(k*k);
   par[i0+4].drv:=par[i0+4].drv+wlat*(l*l);
   par[i0+5].drv:=par[i0+5].drv+wlat*(k*l);
   par[i0+6].drv:=par[i0+6].drv+wlat*(h*l);
   par[i0+7].drv:=par[i0+7].drv+wlat*(h*k);
   END;
          {DERIVATIVES WITH RESPECT TO WIDTH}
   WITH gfit DO
   CASE keyWdt OF
   0,1: BEGIN par[iwd].drv:=par[iwd].drv-wcc*z; {W0}
        par[iwd+1].drv:=par[iwd+1].drv-wcc*z*xm; {W1}
        IF(NOT tofa)THEN
         par[iwd+2].drv:=par[iwd+1].drv-wcc*z*SQR(xm2); {W2}
        END;
   2:   BEGIN wc:=-wcc*z*0.5/zn;
        par[iwd+1].drv:=par[iwd+1].drv+wc/q_old; {W1}
        par[iwd].drv:=par[iwd].drv+wc;  {W0}
        END;
   ELSE END;

   IF(anis_mod)THEN WITH gfit DO {Ders to aniso-pars}
    BEGIN FOR i:=1 TO 6 DO
     par[iw+3+i].drv:=par[iw+3+i].drv-wcc*z*aww[i];
    FOR i:=1 TO npa2 DO par[iw+9+i].drv:=par[iw+9+i].drv-wcc*z*aw2[i];
    r:=cph*Fk2*mn*facL;
    FOR i:=1 TO 3 DO par[iw+i].drv:=par[iw+i].drv+r*awa[i]*(ff+anic);
    END;
          {DERIVATIVES WITH RESPECT TO LPATH,D0,LAMBDA,dTHETA}
  WITH gfit DO
   BEGIN cnst:=C252/q12; r:=radian;
   WITH spec[numsp] DO
   CASE tofa OF
   true: BEGIN par[j0+1].drv:=par[j0+1].drv-wcc*cnst*sinT;{der to LPath}
         par[j0+3].drv:=par[j0+3].drv+wcc; {der to D0}
         END;
   ELSE  BEGIN
         par[j0+3].drv:=par[j0+3].drv+wcc*celx1;{der to D0}
         par[j0+1].drv:=par[j0+1].drv-wcc*q12*r*celx1/fq2;{dr to lambda}
         END;
       END;
   par[j0+2].drv:=
    par[j0+2].drv-wcc*cnst*COS((theta0)/r)*Lpath/r;{der to dTh0}
   END;

          {PHASE VALUE}
  phsum:=phsum+mn*facL*Fk2*ania*(ff+anic); {the phase}

          {POWDER MATCH}
  IF(mriamode=2)THEN WITH gfit DO
   BEGIN {L} par[1].drv:=0; {Theta0} par[2].drv:=0;
   {der to StrFac} par[i0+10+iref].drv:=akcm;
   END;

  {der to m" pars}IF(use_m2)THEN WITH gfit DO
   BEGIN r:=cph*Fk2*mn*facL*ania*fzz; par[im2+1].drv:=par[im2+1].drv+r;
   par[im2+2].drv:=par[im2+2].drv+xm*r;
   END;
  L2:;
  END;
                     { REFLECTION loop end}

 sumph:=sumph+cph*phsum; rb0[iph+2]:=cph*phsum;
 {der to Ci} gfit.par[icp+1].drv:=phsum*fcor;
 END;
                    { PHASE loop end}

{der to acor} gfit.par[j0+10].drv:=sumph*LN(xcor);
{store the function value} v:=sumph+backv; rb0[2]:=v;

IF(typ=3)THEN
 BEGIN frp[ksp]:=frp[ksp]+ABS(yy-v);
 frb[ksp]:=frb[ksp]+ABS(yy-backv);
 frw[ksp]:=frw[ksp]+w*SQR(yy-v);
 IF(w<>0)THEN fre[ksp]:=fre[ksp]+SQRT(1/w);
 fie[ksp]:=fie[ksp]+ABS(yy);
 fi2[ksp]:=fi2[ksp]+w*SQR(yy);
 END;
  IF(debugV.aniso)THEN
  {DEBUG}Writeln(fout,'Exit ',ip:7);
END;
PROCEDURE ConExAr(VAR yy,ww,v:REAL; jp,typ:INTEGER);
VAR   z,R,fu,df,x,A,B,sig,T,t_0:REAL; i:INTEGER;
LABEL L2;
 FUNCTION ex(x:REAL):REAL;
 BEGIN ex:=EXP(-x/T);
 END;
 FUNCTION ef(x,sig:REAL):REAL;
 BEGIN ef:=erf(x/sig);
 END;
BEGIN x:=jp-1; WITH gfit DO
 BEGIN B:=par[1].p; A:=par[2].p; T:=par[3].p; sig:=par[4].p;
 t_0:=par[5].p; R:=sig*sig/T;
 par[1].drv:=1;{to background}
 FOR i:=2 TO 5 DO par[i].drv:=0; v:=0; IF(x>=t_0)THEN
  CASE morozm OF
  0: BEGIN v:=A*sig*ex(x)*ef(x,sig);
     par[2].drv:=sig*ex(x)*ef(x,sig); {to amplitude}
     par[3].drv:=A*sig*ex(x)*ef(x,sig)*x/SQR(T); {to halflife}
     par[4].drv:=
      (A*(sig+0.0001)*ex(x)*ef(x,sig+0.0001)-v)/0.0001; {to sig}
     par[5].drv:=0;
     END;
  1: BEGIN z:=(x-R-t_0)/sig; myerf(z,fu,df); v:=A*ex(x)*fu/T;
     par[2].drv:=ex(x)*fu/T; {to amplitude}
     par[3].drv:=A/SQR(T)/T*(sig*ex(x)*df+(x-T)*fu); {to halflife}
     par[4].drv:=0;
     par[5].drv:=-A/sig*ex(x)*df/T; {to sig}
     END;
  ELSE END;
 rb0[1]:=B;
 rb0[2]:=B;
 rb0[3]:=B+v;
 v:=v+B;
 END;
yy:=spec[numsp].y[jp];
ww:=spec[numsp].e[jp]; IF(ww<>0)THEN ww:=1/(ww*ww);
END;
PROCEDURE arpeak(VAR yy,w,v:REAL; jp,typ:INTEGER);
VAR   i,j,ia,ip,iw,id,jm,ips:INTEGER;
      xm,x1,x2,x,z,z2,rr,rrr,dif,zn,ac,g1,xfz2,zfz2,fzg12:REAL;
      am2,ff,fz,fzz,fzzz,xfz,zfz,fzg1,ps0,ps1,dps0,dps1:REAL;
      fw,fd:ARRAY[1..6] OF REAL;
LABEL L3,L9;
BEGIN i:=jp+spec[numsp].a11-1;
x:=i; IF(neqexp)THEN x:=spec[numsp].x[i]; IF(typ=4)THEN x:=xarg;
x1:=spec[numsp].a11; x2:=spec[numsp].a22; xm:=(2*x-x1-x2)/(x2-x1);
rr:=0; j:=Round(x);
yy:=spec[numsp].y[j]; w:=spec[numsp].e[j]; IF(w<>0)THEN w:=1/(w*w);
FOR i:=1 TO gfit.npar DO gfit.par[i].drv:=0;
IF(use_m2)THEN WITH gfit DO
 BEGIN ips:=GetNAddiPar(2); ps0:=par[ips+1].p; ps1:=par[ips+2].p;
 END;
{background parameter processing}
ProcessBackground(-3,rr,x,xm);
rb0[1]:=rr; IF(npeak=0)THEN GOTO L9;
{peak parameter processing}
L3: FOR j:=1 TO 4 DO
 BEGIN fw[j]:=0; fd[j]:=0;
 END;
iw:=6+2*npeak; id:=iw+4; dps0:=0; dps1:=0;
FOR i:=1 TO npeak DO WITH gfit DO {loop on components}
 BEGIN jm:=tam[i]; ia:=6+i; ip:=ia+npeak;
 dif:=x-par[ip].p; zn:=par[iw+jm].p+par[id+jm].p*xm;
 zn:=AMAX1(zn,0.1); ac:=par[ia].p; g1:=1/zn; z:=dif*g1;
 ff:=0; fz:=0; fzz:=0; fzzz:=0;
 WITH pmod[jm] DO IF(ABS(z)<rl{cpw})THEN
  BEGIN IF(use_m2)THEN calf3(z,ff,fz,fzz,fzzz)
                  ELSE Getffzfzz(z,ff,fz,fzz);
  END;
 {the 1st ders}
 rrr:=ac*ff; fzg1:=-fz*g1; zfz:=ac*fzg1*z; xfz:=zfz*xm;
 par[ia].drv:=ff; par[ip].drv:=ac*fzg1;
 fw[jm]:=fw[jm]+zfz; fd[jm]:=fd[jm]+xfz;
 IF(use_m2)THEN
  BEGIN am2:=ps0+ps1*xm; fzg12:=-fzzz*g1; zfz2:=am2*fzg12*z;
  xfz2:=zfz2*xm; rrr:=rrr+am2*fzz;
  dps0:=dps0+fzz; dps1:=dps1+xm*fzz;
  par[ip].drv:=par[ip].drv+am2*fzg12;
  fw[jm]:=fw[jm]+zfz2; fd[jm]:=fd[jm]+xfz2;
  END;
 rb0[2+i]:=rrr; rr:=rr+rrr;
 END; {end of loop on components}
{store the 1st ders}
FOR i:=1 TO 4 DO WITH gfit DO
 BEGIN par[iw+i].drv:=fw[i]; par[id+i].drv:=fd[i];
 END;
IF(use_m2)THEN WITH gfit DO
 BEGIN par[ips+1].drv:=dps0; par[ips+2].drv:=dps1;
 END;
L9: v:=rr; rb0[2]:=rr;
END;
PROCEDURE FkDerivatives(iph,iref,i0:INTEGER; akcm,Fkc,Fks,pt,Fkm:REAL);
{ Purpose: Compute the derivatives of the structure factor
         DF2/db = sum_at 2*(cos*Fc+sin*Fs)
         DF2/dx = sum_at 2*(-sin*Fc+cos*Fs)*2pi*h
         DF2/dy = sum_at 2*(-sin*Fc+cos*Fs)*2pi*k
         DF2/dz = sum_at 2*(-sin*Fc+cos*Fs)*2pi*l
         DF2/dt = sum_at
  Version: 18 Jun 1990; delphi: Sep 28 2000  }
VAR i,iii,jjj,iatom,j,h,k,l,it,ic,mu:INTEGER;
    bj,xj,yj,zj,b11,b12,b13,b22,b23,b33,sumb,cscf,rtrans:REAL;
    u11,u12,u13,u21,u22,u23,u31,u32,u33,ht,kt,lt,xtj,ytj,ztj,arg:REAL;
    r,thm,expb,expb0,cosa,sina,akcme,rcpx,rdif,wsina,terma:REAL;
    dfa,dfb,dfc:REAL; tr:ARRAY[1..3] OF REAL;
BEGIN FOR iatom:=1 TO lat[iph].nat DO WITH gfit DO
 BEGIN iii:=iatom*10+i0;
 {FOR i:=1 TO 10 DO IF(par[iii+i].pl<>par[iii+i].pu)THEN} {If par not f}
  BEGIN bj:=par[iii+1].p;
  xj:=par[iii+2].p; yj:=par[iii+3].p; zj:=par[iii+4].p;
  b11:=par[iii+5].p;
  b12:=par[iii+6].p{*2*lat[iph].o23*0.125};
  b13:=par[iii+7].p{*2*lat[iph].o13*0.125};
  b22:=par[iii+8].p{*lat[iph].o22*0.25};
  b23:=par[iii+9].p{*2*lat[iph].o12*0.125};
  b33:=par[iii+10].p{*lat[iph].o33*0.25};
  IF(atoms[iatom,iph].isotr)THEN
   BEGIN thm:=b11*q_old*0.25; expb0:=EXP(-AMIN1(thm,64));
   END                     ELSE
   BEGIN {b11:=b11*lat[iph].o11*0.25};
   END;
  h:=hkla[iref,iph].h; k:=hkla[iref,iph].k; l:=hkla[iref,iph].l;
  mu:=hkla[iref,iph].mul;
  IF(keyma[iph]=1)THEN {Magnetic}
   BEGIN r:=0.25/q_old; WITH atoms[iatom,iph] DO
   cscf:=0.539*SIGN(mgmom)*Ab*EXP(-al*r)+Bb*EXP(-bl*r)+Cb*EXP(-cl*r)+Db;
   GetFmDer(iph,1,1,h,k,l,mu,Fkm,dfa,dfb,dfc);
   END            ELSE cscf:=atoms[iatom,iph].scat; {Normal}
  {Calculate for the equivalents}
  WITH lat[iph] DO FOR it:=1 TO ntlv+1 DO
   BEGIN FOR ic:=1 TO 3 DO {Loop in translations}
    IF(it=1)THEN tr[ic]:=0
            ELSE tr[ic]:=tlv[ic,it-1];
   rtrans:=ntlv+1;
   {Loop in symmetry transformations}
   FOR j:=1 TO nut DO
    BEGIN u11:=ut[1,j]; u12:=ut[2,j]; u13:=ut[3,j]; u21:=ut[4,j];
    u22:=ut[5,j]; u23:=ut[6,j]; u31:=ut[7,j]; u32:=ut[8,j];
    u33:=ut[9,j];
    {Compute the new indices}
    ht:=u11*h+u21*k+u31*l; kt:=u12*h+u22*k+u32*l; lt:=u13*h+u23*k+u33*l;
    {Compute the new coordinates}
    xtj:=u11*xj+u12*yj+u13*zj+shv[1,j];
    ytj:=u21*xj+u22*yj+u23*zj+shv[2,j];
    ztj:=u31*xj+u32*yj+u33*zj+shv[3,j];
    {compute the imaginary exponential argument}
    arg:=pi2*(xtj*h+ytj*k+ztj*l);
    {compute the thermal factor}
    IF(atoms[iatom,iph].isotr)THEN expb:=expb0
                              ELSE
     BEGIN WITH lat[iph] DO
      BEGIN ht:=ht/ascx; kt:=kt/bscx; lt:=lt/cscx;
      END;
     thm:=(b11*ht+b12*kt+b13*lt)*ht+(b22*kt+b23*lt)*kt+b33*lt*lt;
     expb:=EXP(-AMIN1(thm,64));
     END;
    cosa:=COS(arg); sina:=SIN(arg); cosa:=cscf*cosa; sina:=cscf*sina;
    akcme:=2*akcm*expb*rtrans;
    {compute sum and difference of COS and SIN}
    IF(csym=0)THEN
     BEGIN rcpx:=cosa*Fkc+sina*Fks; rdif:=-cosa*Fks+sina*Fkc;
     END      ELSE
     BEGIN rcpx:=2*cosa*Fkc; rdif:=2*sina*Fkc;
     END;
    rcpx:=rcpx*pt*SQR(Fkm); rdif:=rdif*pt*SQR(Fkm);
    par[iii+1].drv:=par[iii+1].drv+akcme*rcpx;  {der  to bj}
    wsina:=akcme*rdif*bj*pi2;           {der  to atom coordinates}
    par[iii+2].drv:=par[iii+2].drv-wsina*ht;
    par[iii+3].drv:=par[iii+3].drv-wsina*kt;
    par[iii+4].drv:=par[iii+4].drv-wsina*lt;
    terma:=akcme*rcpx*bj;               {der  to thermal tensor}
   IF(atoms[iatom,iph].isotr)THEN
    BEGIN par[iii+5].drv:=par[iii+5].drv-terma*q_old*0.25;
    par[iii+6].drv:=0; par[iii+7].drv:=0; par[iii+8].drv:=0;
    par[iii+9].drv:=0; par[iii+10].drv:=0;
    END                      ELSE
     BEGIN par[iii+5].drv:=par[iii+5].drv-terma*(ht*ht);
     par[iii+6].drv:=par[iii+6].drv-terma*(ht*kt)*2;
     par[iii+7].drv:=par[iii+7].drv-terma*(ht*lt)*2;
     par[iii+8].drv:=par[iii+8].drv-terma*(kt*kt);
     par[iii+9].drv:=par[iii+9].drv-terma*(kt*lt)*2;
     par[iii+10].drv:=par[iii+10].drv-terma*(lt*lt);
     END;
    END;
   {BREAK;} {Break since check for one not fixed atom enough}
   END;
  END;
 IF(keyma[iph]=1)THEN WITH lat[iph] DO WITH gfit DO {Magnetic}
  BEGIN jjj:=GetNAddiPar(0)+(iph-1)*3; r:=akcme*(Fkc*Fkc+Fks*Fks)/2;
  par[jjj+1].drv:=par[jjj+1].drv-r*2*SQRT(1-Fkm*Fkm)*Fkm*dfa;
  par[jjj+2].drv:=par[jjj+2].drv-r*2*SQRT(1-Fkm*Fkm)*Fkm*dfb;
  par[jjj+3].drv:=par[jjj+3].drv-r*2*SQRT(1-Fkm*Fkm)*Fkm*dfc;
  {Writeln(fout,drv[jjj+1]:9:5,drv[jjj+2]:9:5,drv[jjj+3]:9:5);
  Writeln(fout,iref:5,akcme:9:2,
  dfa:9:5,dfb:8:2,dfc:9:5,Fkc:8:2,Fkm:8:2);}
  END;
 END;
END;
PROCEDURE UpdateOldCell(iph:INTEGER);
VAR i0,j:INTEGER;
BEGIN WITH lat[iph] DO WITH gfit DO
 BEGIN i0:=Geti0(iph); o11:=par[i0+2].p; o22:=par[i0+3].p;
 o33:=par[i0+4].p; o12:=par[i0+7].p; o13:=par[i0+6].p; o23:=par[i0+5].p;
 IF(keyma[iph]=1)THEN
  BEGIN j:=GetNAddiPar(0)+(iph-1)*3+1;
  h0:=par[j].p; k0:=par[j+1].p; l0:=par[j+2].p;
  END;
 END;
END;
PROCEDURE regmria(iit:INTEGER);
{A special regularizer program for displaying intermediate fit results}
VAR s,s1:STRING; iph:INTEGER;
BEGIN ConvertStr(gfit.rb.chi2,25,2); Compress(sedz);
s:=' Iteration= '+IntToStr(gfit.rb.gnit); s1:='         Chi2= '+sedz;
FOR iph:=1 TO nph DO UpdateOldCell(iph); {Update old unit cell}
IF(dispbut)THEN
 BEGIN L0rec:=modestr[mriamode]; Form2.Timer1.Interval:=1; Wait(click1);
 OuttoLabel(Form2.Label5,15,30,clGreen,clWhite,s);
 OuttoLabel(Form2.Label6,50,50,clGreen,clAqua,s1);
 END       ELSE Form2.Caption:=s+s1;
END;
PROCEDURE ProcessBackground(ibk:INTEGER; VAR backv:REAL; d,xm:REAL);
{Purpose: To process background parameters;   Version: Dec 2 1997
 ibk=shift of parameters; backv=value to be got, d=x, xm=(x-x-m)/(i2-i1)
}
VAR   i,i1,i2:INTEGER;
      z,ff,fz,fzz,wx,z2,zz,pn,pnff:REAL;
LABEL L20,Lend;
BEGIN z:=xm; z2:=z*z; gfit.par[ibk+4].drv:=1; backv:=0;
i1:=ibk+4; i2:=ibk+9; IF(keyBck=1)THEN GOTO L20;
{compute for the polynomial background case} WITH gfit DO
 BEGIN par[ibk+5].drv:=z;
 par[ibk+6].drv:=1.5*z2-0.5;
 par[ibk+7].drv:=(2.5*z2-1.5)*z;
 par[ibk+8].drv:=((35*z2-30)*z2+3)*0.125;
 par[ibk+9].drv:=(((63*z2-70)*z2+15)*z)*0.125;
 FOR i:=i1 TO i2 DO backv:=backv+par[i].p*par[i].drv; GOTO Lend;
 {compute for case of real background}
 L20: wx:=AMAX1(1,par[ibk+8].p+par[ibk+9].p*z);
 zz:=(d-par[ibk+7].p)/wx; pn:=par[ibk+5].p+par[ibk+6].p*z2;
 pmod[0].calf(zz,ff,fz,fzz);
 par[ibk+5].drv:=ff; par[ibk+6].drv:=ff*z2;
 pnff:=pn*fz; par[ibk+7].drv:=-pnff/wx;
 par[ibk+8].drv:=par[ibk+7].drv*zz; par[ibk+9].drv:=par[ibk+8].drv*z;
 backv:=(pn*ff+par[ibk+4].p);
 END;
{Save the background value}
Lend: rb0[1]:=backv;
END;
PROCEDURE ComputeQs(i0,iph,iref:INTEGER; VAR q,qold:REAL);
{ Purpose: To compute q and qold;   Version: Oct 21 1997;  Oct 7 2000}
{  q_hkl = (ah)^2 + (bk)^2 + (cl)^2 + 2bcklCOSa + 2achlCOSb + 2abhkCOSg
   d_hkl = 1/SQRT(q_hkl)}
BEGIN WITH lat[iph] DO WITH hkla[iref,iph] DO qold:=
 (o11*h+o12*k+o13*l)*h+(o22*k+o23*l)*k+o33*l*l;
WITH gfit DO WITH hkla[iref,iph] DO q:=(par[i0+2].p*h+par[i0+7].p*k+
 par[i0+6].p*l)*h+(par[i0+3].p*k+par[i0+5].p*l)*k+par[i0+4].p*l*l;
END;
PROCEDURE GetSingleFk(j,iph:INTEGER; VAR rc,rs:REAL);
VAR i0:INTEGER; phi,r,rr:REAL;
BEGIN i0:=Geti0(iph)+10; r:=SQRT(ABS(gfit.par[i0+j].p));
WITH hkla[j,iph] DO
 IF(lat[iph].csym=1)THEN {Symm.case}
  BEGIN rc:=r; IF(Fkc<0)THEN rc:=-r; rs:=0;
  END               ELSE {Non-symm.case, complex number}
  BEGIN rr:=ABS(Fkc)/AMAX1(ABS(Fks),1E-5); phi:=Arctan(rr);
  IF((Fkc>0)AND(Fks<0))THEN phi:=-phi;
  IF((Fkc<0)AND(Fks>0))THEN phi:=pi*0.5-phi;
  IF((Fkc<0)AND(Fks<0))THEN phi:=phi+pi*0.5;
  rc:=r*COS(phi); rs:=r*SIN(phi);
  END;
END;
PROCEDURE coextc(pj0,theta:REAL; iref,iph:INTEGER; VAR Yp,dFdrho:REAL);
{Purpose: To compute the primary extinction, Becker & Coppens,1974
          Primary extinction from Zachariasen W.H.(1967) (revised)
          Version: 11.03.1990  }
VAR sin2t,gamma,Xp,Fk2old,flame:REAL;
BEGIN { Becker & Coppens version for primary extinction correction}
{cos2T:=COS(2*theta/radian);
ap:=0.2+.45*cos2T;
bp:=0.22-0.12*(.5-cos2T)**2;
   Primary extinction from Becker & Coppens(1974)
Fk2old:=hkla(5,iref,iph)**2+hkla(6,iref,iph)**2;
flame:=2.0*SIN(theta/radian)/SQRT(4.*sdkold);
gamma:=(volum(iph)*flame**4)*Fk2old*.01;
Xp:=gamma*p(j0);
bca:=AMAX1(ap*Xp*Xp/(1.+bp*Xp),0.);
Yp:=1./SQRT(1.+2.*Xp+bca);
dYp:=-0.5*Yp**3*(2.+bca*ap*Xp*(2.+bp*Xp)/(1.+bp*Xp)**2);
dFdrho:=gamma*dYp;  }
{ Zachariasen W.H. version  }
Fk2old:=SQR(hkla[iref,iph].Fkc)+SQR(hkla[iref,iph].Fks);
flame:=SIN(theta/radian)/SQRT(q_old);
{ 2 is absent since sdkold is 4 times less  }
sin2t:=SIN(2*theta/radian);
gamma:=(powerRI(flame,3)*Fk2old)/(SQRT(lat[iph].vol)*sin2t);
Xp:=gamma*pj0;
Yp:=1/AMAX1(SQRT(ABS(1+2*Xp)),0.001);
dFdrho:=-powerRI(Yp,3)*gamma;
END;
PROCEDURE MakeFitResLB(VAR i:INTEGER);
BEGIN WITH LB1rec DO WITH gfit DO
 BEGIN ar[i]:=' '; Inc(i); ar[i]:=compstr; Inc(i);
 IF(nocnv)THEN
  BEGIN ar[i]:=gf_pnc; Inc(i);
  END;
 IF(noinv)THEN
  BEGIN ar[i]:=gf_mni; Inc(i);
  END;
 IF((noch)OR(noequ))THEN
  BEGIN ar[i]:=gf_fng; Inc(i);
  END;
 {IF(noeps)THEN
  BEGIN ar[i]:=gf_anr; Inc(i);
  END;}
 IF(rb.pobi=1)THEN
  BEGIN ar[i]:=gf_phb; Inc(i);
  END;
 ar[i]:=' '; Inc(i);
 ConvertStr(rb.chi2,8,2); s:='  Ch2/df= '+sedz;
 ConvertStr(rb.chipl,5,2); s:=s+'     Ch+= '+sedz; ar[i]:=s; Inc(i);
 ConvertStr(rb.epsf,6,1); s:='  Eps= '+sedz;
 ConvertStr(Nittot,5,0); s:=s+'       Iterations= '+sedz;
 ar[i]:=s; nitems:=i;
 END;
END;
PROCEDURE MakeBMGraph(amx0,amn0:REAL; indi:STRING);
{To make a bitmap graph}
VAR   i,j,k,np,ns,cou:INTEGER; amxB,xx,x,st,z,r,v,v0,backf:REAL;
      ii,jj,li,lj,lix,ix1,iy1,ix2,iy2:INTEGER; mongr:BOOLEAN;
BEGIN np:=ic2-ic1+1; st:=512/AMAX0(1,np); cou:=MAX0(1,np DIV 10);
amxB:=amx0+SQRT(ABS(amx0)); j:=0; FOR i:=1 TO 5 DO IF(indi[i]='x')THEN
 BEGIN Inc(j);
 END;
IF(j=1)THEN mongr:=true ELSE mongr:=false; MakeBMFrame(amxB,amn0);
           {Plotting the Graph to show how graph proceeds}
ns:=numsp; WITH Form3 DO
 BEGIN Left:=250; Top:=250; Width:=150; Height:=50; Show;
 Memo1.Visible:=false; Button1.Visible:=false; Label1.Visible:=false;
 END;
WITH Bitmap.Canvas DO
 BEGIN Pen.Style:=psSolid; Pen.Color:=clAqua; FOR i:=ic1 TO ic2 DO
  BEGIN x:=i-ic1+1; j:=Round(x*st); li:=exs(j);
  IF(xscale)THEN li:=exs(Round((LN(FLOAT(li)+1)/LN(513))*512));
  IF(mongr)THEN              {MODE = one spectrum}
           {Exp.sp.}
   BEGIN lj:=exs({y1+}CalcOrdinate(spec[ns].y[i],amxB,amn0));
   DrawPixelPr(clFuchsia,li,lj,i,ic1,1);
            {Fit}
   IF(grst=2)THEN
    BEGIN Pen.Color:=clAqua; FOR ii:=0 TO 2 DO
     BEGIN r:=ii*0.3333; xx:=i-ic1+1+r; xarg:=xx+ic1-1; sarg:=numsp;
     IF(neqexp)THEN xarg:=spec[ns].x[i]+r; backf:=0; v:=0; k:=Round(xx);
     q1:=r;
     WITH spec[numsp] DO IF(infun(i,a11,a22))THEN
      BEGIN CASE mriamode OF
      0,2: armrit(r,r,v,k,4);
      3:   IF(positexp)THEN ConExAr(q2,r,v,k,4) ELSE arpeak(r,r,v,k,4);
            ELSE END;
      backf:=rb0[1];
      END;
     lj:=exs({y1+}CalcOrdinate(v,amxB,amn0));
     lix:=exs(Round(xx*st)+1); IF(ii=0)THEN v0:=v;
     IF(i=ic1)THEN
      BEGIN ix1:=lix; iy1:=lj;
      END     ELSE
       BEGIN ix2:=lix; iy2:=lj; Moveto(ix1,iy1); Lineto(ix2,iy2);
       ix1:=ix2; iy1:=iy2;
       END;
     END;
            {Background}
    lj:=exs({y1+}CalcOrdinate(backf,amxB,amn0));
    Pixels[li,lj]:=clBlue;
    Pixels[li-1,lj]:=clBlue; Pixels[li+1,lj]:=clBlue;
             {Zero+Errors}
    lj:=exs({y1+}CalcOrdinate(0,amxB,amn0)); Pixels[li,lj]:=clLime;
    IF(ErrGrM)THEN
     BEGIN ii:=Round({10*}(spec[ns].y[i]-v0)/AMAX1(spec[ns].e[i],1));
     IF(spec[ns].e[i]=0)THEN ii:=0;
     IF(ii>7)THEN ii:=7; IF(ii<-7)THEN ii:=-7; ii:=267-ii;
     lj:=exs(ii); DrawPixelPr(clRed,li,lj,i,ic1,5);
     END;

    END;
   END     ELSE              {MODE = Impose several spectra}
    FOR k:=1 TO 5 DO IF(indi[k]='x')THEN
     BEGIN lj:=exs({y1+}CalcOrdinate(spec[k-1].y[i],amxB,amn0));
     DrawPixelPr(cola[k],li,lj,i,ic1,k);
     END;
  IF(((i-ic1)MOD cou)=0)THEN
   BEGIN Form3.Caption:='*   '+IntToStr(((i-ic1) DIV cou)*10)+'%';
   END;
  END;  {End loop in points}
 END;
Form3.Close;
END;


{---------------------}
BEGIN
END.



     {  q52,q32,r_a,r_b,r_c,r_al,r_be,r_ga:REAL;
     WITH lat[iph].lpi DO
      BEGIN r_a:=a*ht*ht+b*gamma*ht*kt+c*beta*ht*lt;
      r_b:=b*kt*kt+a*gamma*ht*kt+c*alpha*kt*lt;
      r_c:=c*lt*lt+a*beta*ht*lt+b*alpha*kt*lt;
      r_al:=b*c*kt*lt;
      r_be:=a*c*ht*lt;
      r_ga:=a*b*ht*kt;
      q52:=q_old*q_old*SQRT(q_old);
      q32:=q_old*SQRT(q_old); z:=SQR(r_al)+SQR(r_be)+SQR(r_ga);
      aww[1]:=aww[1]+(3*r_a*r_a/q52-ht*ht/q32)*mul/r;
      aww[2]:=aww[2]+(3*r_b*r_b/q52-kt*kt/q32)*mul/r;
      aww[3]:=aww[3]+(3*r_c*r_c/q52-lt*lt/q32)*mul/r;
      aww[4]:=aww[4]+(3*z/q52)*mul/r;
      awa[1]:=awa[1]+SQR(r_a/q32);
      awa[2]:=awa[2]+SQR(r_b/q32);
      awa[3]:=awa[3]+SQR(r_c/q32);
      END;
     END;}

{FOR i:=1 TO nsp-1 DO
 BEGIN ii:=spec[i].a22-spec[i].a11+1;
 IF(j<ii)THEN CONTINUE ELSE Inc(ksp); j:=j-ii;
 END;}
   {IF(use_m2)THEN
    BEGIN calf(z,ff,fz,fzz); pmod[0].calf(z,ffu,fzu,r);
    END      ELSE}
        {IF(use_m2)THEN
         BEGIN drv[iwd]:=drv[iwd]-wca; drv[iwd+1]:=drv[iwd+1]-wca*xm;
         drv[iwd+2]:=drv[iwd+2]+drv[iwd+1]-wca*z*SQR(xm);
         END;}
    {r:=lat[iph].hklx2*3*d3;
    aw2[1]:=h*h/r; aw2[2]:=k*k/r; aw2[3]:=l*l/r;}
      {aww[i]:=aww[i]/(d3+d5);}
    {d3:=q_old*SQRT(q_old);}


